@@ -3,7 +3,7 @@ import random
 import time
 
 
-class BoundedSet(object):
+class BoundedSet:
     """A set with a maximum size that evicts the oldest items when necessary.
 
     This class does not implement the complete set interface.
@@ -27,7 +27,7 @@ class BoundedSet(object):
         self._set.add(item)
 
 
-class ExponentialCounter(object):
+class ExponentialCounter:
     """A class to provide an exponential counter with jitter."""
 
     def __init__(self, max_counter):
@@ -41,7 +41,7 @@ class ExponentialCounter(object):
 
     def counter(self):
         """Increment the counter and return the current value with jitter."""
-        max_jitter = self._base / 16.
+        max_jitter = self._base / 16.0
         value = self._base + random.random() * max_jitter - max_jitter / 2
         self._base = min(self._base * 2, self._max)
         return value
@@ -67,17 +67,22 @@ def permissions_string(permissions, known_permissions):
     """
     to_set = []
     if permissions is None:
-        to_set = ['+all']
+        to_set = ["+all"]
     else:
-        to_set = ['-all']
+        to_set = ["-all"]
         omitted = sorted(known_permissions - set(permissions))
-        to_set.extend('-{}'.format(x) for x in omitted)
-        to_set.extend('+{}'.format(x) for x in permissions)
-    return ','.join(to_set)
+        to_set.extend("-{}".format(x) for x in omitted)
+        to_set.extend("+{}".format(x) for x in permissions)
+    return ",".join(to_set)
 
 
-def stream_generator(function, pause_after=None, skip_existing=False,
-                     attribute_name='fullname'):
+def stream_generator(
+    function,
+    pause_after=None,
+    skip_existing=False,
+    attribute_name="fullname",
+    **function_kwargs
+):
     """Yield new items from ListingGenerators and ``None`` when paused.
 
     :param function: A callable that returns a ListingGenerator, e.g.
@@ -97,6 +102,8 @@ def stream_generator(function, pause_after=None, skip_existing=False,
 
     :param attribute_name: The field to use as an id (default: "fullname").
 
+    Additional keyword arguments will be passed to ``function``.
+
     .. note:: This function internally uses an exponential delay with jitter
        between subsequent responses that contain no new results, up to a
        maximum delay of just over a 16 seconds. In practice that means that the
@@ -167,8 +174,15 @@ def stream_generator(function, pause_after=None, skip_existing=False,
         if before_attribute is None:
             limit -= without_before_counter
             without_before_counter = (without_before_counter + 1) % 30
-        for item in reversed(list(function(
-                limit=limit, params={'before': before_attribute}))):
+        for item in reversed(
+            list(
+                function(
+                    limit=limit,
+                    params={"before": before_attribute},
+                    **function_kwargs
+                )
+            )
+        ):
             attribute = getattr(item, attribute_name)
             if attribute in seen_attributes:
                 continue
