@@ -1,25 +1,36 @@
 """Provide the Reddit class."""
+import configparser
 import os
+from itertools import islice
 
 try:
     from update_checker import update_check
+
     UPDATE_CHECKER_MISSING = False
 except ImportError:  # pragma: no cover
     UPDATE_CHECKER_MISSING = True
 
 
-from prawcore import (Authorizer, DeviceIDAuthorizer, ReadOnlyAuthorizer,
-                      Redirect, Requestor, ScriptAuthorizer,
-                      TrustedAuthenticator, UntrustedAuthenticator, session)
+from prawcore import (
+    Authorizer,
+    DeviceIDAuthorizer,
+    ReadOnlyAuthorizer,
+    Redirect,
+    Requestor,
+    ScriptAuthorizer,
+    TrustedAuthenticator,
+    UntrustedAuthenticator,
+    session,
+)
 
-from .exceptions import ClientException
-from .config import Config
-from .const import __version__, API_PATH, USER_AGENT_FORMAT, configparser
-from .objector import Objector
 from . import models
+from .config import Config
+from .const import __version__, API_PATH, USER_AGENT_FORMAT
+from .exceptions import ClientException
+from .objector import Objector
 
 
-class Reddit(object):
+class Reddit:
     """The Reddit class provides convenient access to reddit's API.
 
     Instances of this class are the gateway to interacting with Reddit's API
@@ -59,8 +70,10 @@ class Reddit(object):
         if value:
             self._core = self._read_only_core
         elif self._authorized_core is None:
-            raise ClientException('read_only cannot be unset as only the '
-                                  'ReadOnlyAuthorizer is available.')
+            raise ClientException(
+                "read_only cannot be unset as only the "
+                "ReadOnlyAuthorizer is available."
+            )
         else:
             self._core = self._authorized_core
 
@@ -70,10 +83,14 @@ class Reddit(object):
 
     def __exit__(self, *_args):
         """Handle the context manager close."""
-        pass
 
-    def __init__(self, site_name=None, requestor_class=None,
-                 requestor_kwargs=None, **config_settings):
+    def __init__(
+        self,
+        site_name=None,
+        requestor_class=None,
+        requestor_kwargs=None,
+        **config_settings
+    ):  # noqa: D207, D301
         """Initialize a Reddit instance.
 
         :param site_name: The name of a section in your ``praw.ini`` file from
@@ -90,7 +107,7 @@ class Reddit(object):
             used to initialize the requestor (default: None).
 
         Additional keyword arguments will be used to initialize the
-        :class`.Config` object. This can be used to specify configuration
+        :class:`.Config` object. This can be used to specify configuration
         settings during instantiation of the :class:`.Reddit` instance. For
         more details please see :ref:`configuration`.
 
@@ -101,9 +118,13 @@ class Reddit(object):
         * user_agent
 
         The ``requestor_class`` and ``requestor_kwargs`` allow for
-        customization of the requestor :class`.Reddit` will use. This allows,
+        customization of the requestor :class:`.Reddit` will use. This allows,
         e.g., easily adding behavior to the requestor or wrapping its
-        :class`Session` in a caching layer. Example usage:
+        |Session|_ in a caching layer. Example usage:
+
+        .. |Session| replace:: ``Session``
+        .. _Session: https://2.python-requests.org/en/master/api/\
+#requests.Session
 
         .. code-block:: python
 
@@ -125,34 +146,42 @@ class Reddit(object):
         self._unique_counter = 0
 
         try:
-            config_section = site_name or os.getenv('praw_site') or 'DEFAULT'
+            config_section = site_name or os.getenv("praw_site") or "DEFAULT"
             self.config = Config(config_section, **config_settings)
         except configparser.NoSectionError as exc:
-            help_message = ('You provided the name of a praw.ini '
-                            'configuration which does not exist.\n\nFor help '
-                            'with creating a Reddit instance, visit\n'
-                            'https://praw.readthedocs.io/en/latest/code_overvi'
-                            'ew/reddit_instance.html\n\n'
-                            'For help on configuring PRAW, visit\n'
-                            'https://praw.readthedocs.io/en/latest/getting_sta'
-                            'rted/configuration.html')
+            help_message = (
+                "You provided the name of a praw.ini "
+                "configuration which does not exist.\n\nFor help "
+                "with creating a Reddit instance, visit\n"
+                "https://praw.readthedocs.io/en/latest/code_overvi"
+                "ew/reddit_instance.html\n\n"
+                "For help on configuring PRAW, visit\n"
+                "https://praw.readthedocs.io/en/latest/getting_sta"
+                "rted/configuration.html"
+            )
             if site_name is not None:
-                exc.message += '\n' + help_message
+                exc.message += "\n" + help_message
             raise
 
-        required_message = ('Required configuration setting {!r} missing. \n'
-                            'This setting can be provided in a praw.ini file, '
-                            'as a keyword argument to the `Reddit` class '
-                            'constructor, or as an environment variable.')
-        for attribute in ('client_id', 'user_agent'):
-            if getattr(self.config, attribute) in (self.config.CONFIG_NOT_SET,
-                                                   None):
+        required_message = (
+            "Required configuration setting {!r} missing. \n"
+            "This setting can be provided in a praw.ini file, "
+            "as a keyword argument to the `Reddit` class "
+            "constructor, or as an environment variable."
+        )
+        for attribute in ("client_id", "user_agent"):
+            if getattr(self.config, attribute) in (
+                self.config.CONFIG_NOT_SET,
+                None,
+            ):
                 raise ClientException(required_message.format(attribute))
         if self.config.client_secret is self.config.CONFIG_NOT_SET:
-            raise ClientException(required_message.format('client_secret') +
-                                  '\nFor installed applications this value '
-                                  'must be set to None via a keyword argument '
-                                  'to the `Reddit` class constructor.')
+            raise ClientException(
+                required_message.format("client_secret")
+                + "\nFor installed applications this value "
+                "must be set to None via a keyword argument "
+                "to the `Reddit` class constructor."
+            )
 
         self._check_for_update()
         self._prepare_objector()
@@ -219,6 +248,19 @@ class Reddit(object):
 
         """
 
+        self.redditors = models.Redditors(self, None)
+        """An instance of :class:`.Redditors`.
+
+        Provides the interface for Redditor discovery. For example
+        to iterate over the newest Redditors, run:
+
+        .. code-block:: python
+
+           for redditor in reddit.redditors.new(limit=None):
+               print(redditor)
+
+        """
+
         self.subreddit = models.SubredditHelper(self, None)
         """An instance of :class:`.SubredditHelper`.
 
@@ -279,25 +321,44 @@ class Reddit(object):
             Reddit.update_checked = True
 
     def _prepare_objector(self):
-        self._objector = Objector(self)
-        mappings = {self.config.kinds['comment']: models.Comment,
-                    self.config.kinds['message']: models.Message,
-                    self.config.kinds['redditor']: models.Redditor,
-                    self.config.kinds['submission']: models.Submission,
-                    self.config.kinds['subreddit']: models.Subreddit,
-                    'LabeledMulti': models.Multireddit,
-                    'Listing': models.Listing,
-                    'LiveUpdate': models.LiveUpdate,
-                    'LiveUpdateEvent': models.LiveThread,
-                    'ModmailAction': models.ModmailAction,
-                    'ModmailConversation': models.ModmailConversation,
-                    'ModmailMessage': models.ModmailMessage,
-                    'UserList': models.RedditorList,
-                    'modaction': models.ModAction,
-                    'more': models.MoreComments,
-                    'stylesheet': models.Stylesheet}
-        for kind, klass in mappings.items():
-            self._objector.register(kind, klass)
+        mappings = {
+            self.config.kinds["comment"]: models.Comment,
+            self.config.kinds["message"]: models.Message,
+            self.config.kinds["redditor"]: models.Redditor,
+            self.config.kinds["submission"]: models.Submission,
+            self.config.kinds["subreddit"]: models.Subreddit,
+            self.config.kinds["trophy"]: models.Trophy,
+            "Button": models.Button,
+            "Collection": models.Collection,
+            "Image": models.Image,
+            "LabeledMulti": models.Multireddit,
+            "Listing": models.Listing,
+            "LiveUpdate": models.LiveUpdate,
+            "LiveUpdateEvent": models.LiveThread,
+            "MenuLink": models.MenuLink,
+            "ModmailAction": models.ModmailAction,
+            "ModmailConversation": models.ModmailConversation,
+            "ModmailMessage": models.ModmailMessage,
+            "Submenu": models.Submenu,
+            "TrophyList": models.TrophyList,
+            "UserList": models.RedditorList,
+            "button": models.ButtonWidget,
+            "calendar": models.Calendar,
+            "community-list": models.CommunityList,
+            "custom": models.CustomWidget,
+            "id-card": models.IDCard,
+            "image": models.ImageWidget,
+            "menu": models.Menu,
+            "modaction": models.ModAction,
+            "moderators": models.ModeratorsWidget,
+            "more": models.MoreComments,
+            "post-flair": models.PostFlairWidget,
+            "stylesheet": models.Stylesheet,
+            "subreddit-rules": models.RulesWidget,
+            "textarea": models.TextArea,
+            "widget": models.Widget,
+        }
+        self._objector = Objector(self, mappings)
 
     def _prepare_prawcore(self, requestor_class=None, requestor_kwargs=None):
         requestor_class = requestor_class or Requestor
@@ -305,8 +366,10 @@ class Reddit(object):
 
         requestor = requestor_class(
             USER_AGENT_FORMAT.format(self.config.user_agent),
-            self.config.oauth_url, self.config.reddit_url,
-            **requestor_kwargs)
+            self.config.oauth_url,
+            self.config.reddit_url,
+            **requestor_kwargs
+        )
 
         if self.config.client_secret:
             self._prepare_trusted_prawcore(requestor)
@@ -314,15 +377,19 @@ class Reddit(object):
             self._prepare_untrusted_prawcore(requestor)
 
     def _prepare_trusted_prawcore(self, requestor):
-        authenticator = TrustedAuthenticator(requestor, self.config.client_id,
-                                             self.config.client_secret,
-                                             self.config.redirect_uri)
+        authenticator = TrustedAuthenticator(
+            requestor,
+            self.config.client_id,
+            self.config.client_secret,
+            self.config.redirect_uri,
+        )
         read_only_authorizer = ReadOnlyAuthorizer(authenticator)
         self._read_only_core = session(read_only_authorizer)
 
         if self.config.username and self.config.password:
             script_authorizer = ScriptAuthorizer(
-                authenticator, self.config.username, self.config.password)
+                authenticator, self.config.username, self.config.password
+            )
             self._core = self._authorized_core = session(script_authorizer)
         elif self.config.refresh_token:
             authorizer = Authorizer(authenticator, self.config.refresh_token)
@@ -331,22 +398,34 @@ class Reddit(object):
             self._core = self._read_only_core
 
     def _prepare_untrusted_prawcore(self, requestor):
-        authenticator = UntrustedAuthenticator(requestor,
-                                               self.config.client_id,
-                                               self.config.redirect_uri)
+        authenticator = UntrustedAuthenticator(
+            requestor, self.config.client_id, self.config.redirect_uri
+        )
         read_only_authorizer = DeviceIDAuthorizer(authenticator)
-        self._core = self._read_only_core = session(read_only_authorizer)
+        self._read_only_core = session(read_only_authorizer)
+        if self.config.refresh_token:
+            authorizer = Authorizer(authenticator, self.config.refresh_token)
+            self._core = self._authorized_core = session(authorizer)
+        else:
+            self._core = self._read_only_core
 
-    def comment(self, id):  # pylint: disable=invalid-name,redefined-builtin
+    def comment(
+        self,  # pylint: disable=invalid-name
+        id=None,  # pylint: disable=redefined-builtin
+        url=None,
+    ):
         """Return a lazy instance of :class:`~.Comment` for ``id``.
 
         :param id: The ID of the comment.
 
+        :param url: A permalink pointing to the comment.
+
         .. note:: If you want to obtain the comment's replies, you will need to
-                  call :meth:`.refresh` on the returned :class:`.Comment`.
+                  call :meth:`~.Comment.refresh` on the returned
+                  :class:`.Comment`.
 
         """
-        return models.Comment(self, id=id)
+        return models.Comment(self, id=id, url=url)
 
     def domain(self, domain):
         """Return an instance of :class:`.DomainListing`.
@@ -364,15 +443,14 @@ class Reddit(object):
             None).
 
         """
-        data = self.request('GET', path, params=params)
+        data = self.request("GET", path, params=params)
         return self._objector.objectify(data)
 
     def info(self, fullnames=None, url=None):
         """Fetch information about each item in ``fullnames`` or from ``url``.
 
-        :param param_list: A list of paramaters, either fullnames for a
-            comment/submission/subreddit or a url for a list of link
-            submissions.
+        :param fullnames: A list of fullnames for comments, submissions, and/or
+            subreddits.
         :param url: A url (as a string) to retrieve lists of link submissions
             from.
         :returns: A generator that yields found items in their relative order.
@@ -381,8 +459,8 @@ class Reddit(object):
         issued in batches for each 100 fullnames.
 
         .. note:: For comments that are retrieved via this method, if you want
-                  to obtain its replies, you will need to call :meth:`.refresh`
-                  on the yielded :class:`.Comment`.
+                  to obtain its replies, you will need to call
+                  :meth:`~.Comment.refresh` on the yielded :class:`.Comment`.
 
         .. note:: When using the URL option, it is important to be aware that
                   URLs are treated literally by Reddit's API. As such, the URLs
@@ -390,30 +468,48 @@ class Reddit(object):
                   different set of submissions.
 
         """
-        if bool(fullnames) == bool(url):
-            raise TypeError('Either `fullnames` or `url` must be provided.')
+        none_count = [fullnames, url].count(None)
+        if none_count > 1:
+            raise TypeError("Either `fullnames` or `url` must be provided.")
+        if none_count < 1:
+            raise TypeError(
+                "Mutually exclusive parameters: `fullnames`, `url`"
+            )
 
-        elif fullnames:
-            if not isinstance(fullnames, list):
-                raise TypeError('fullnames must be a list')
+        if fullnames is not None:
+            if isinstance(fullnames, str):
+                raise TypeError("`fullnames` must be a non-str iterable.")
 
-            def generator():
-                for position in range(0, len(fullnames), 100):
-                    fullname_chunk = fullnames[position:position + 100]
-                    params = {'id': ','.join(fullname_chunk)}
-                    for result in self.get(API_PATH['info'], params=params):
+            def generator(fullnames):
+                iterable = iter(fullnames)
+                while True:
+                    chunk = list(islice(iterable, 100))
+                    if not chunk:
+                        break
+
+                    params = {"id": ",".join(chunk)}
+                    for result in self.get(API_PATH["info"], params=params):
                         yield result
 
-            return generator()
+            return generator(fullnames)
 
-        else:
-            try:
-                params = {'url': url}
-                url_list = [result for result in
-                            self.get(API_PATH['info'], params=params)]
-                return url_list
-            except Exception:
-                raise TypeError('Invalid URL or no posts exist')
+        def generator(url):
+            params = {"url": url}
+            for result in self.get(API_PATH["info"], params=params):
+                yield result
+
+        return generator(url)
+
+    def patch(self, path, data=None):
+        """Return parsed objects returned from a PATCH request to ``path``.
+
+        :param path: The path to fetch.
+        :param data: Dictionary, bytes, or file-like object to send in the body
+            of the request (default: None).
+
+        """
+        data = self.request("PATCH", path, data=data)
+        return self._objector.objectify(data)
 
     def post(self, path, data=None, files=None, params=None):
         """Return parsed objects returned from a POST request to ``path``.
@@ -427,8 +523,20 @@ class Reddit(object):
             None).
 
         """
-        data = self.request('POST', path, data=data or {}, files=files,
-                            params=params)
+        data = self.request(
+            "POST", path, data=data or {}, files=files, params=params
+        )
+        return self._objector.objectify(data)
+
+    def put(self, path, data=None):
+        """Return parsed objects returned from a PUT request to ``path``.
+
+        :param path: The path to fetch.
+        :param data: Dictionary, bytes, or file-like object to send in the body
+            of the request (default: None).
+
+        """
+        data = self.request("PUT", path, data=data)
         return self._objector.objectify(data)
 
     def random_subreddit(self, nsfw=False):
@@ -438,14 +546,15 @@ class Reddit(object):
             (default: False).
 
         """
-        url = API_PATH['subreddit'].format(subreddit='randnsfw' if nsfw
-                                           else 'random')
+        url = API_PATH["subreddit"].format(
+            subreddit="randnsfw" if nsfw else "random"
+        )
         path = None
         try:
-            self.get(url, params={'unique': self._next_unique})
+            self.get(url, params={"unique": self._next_unique})
         except Redirect as redirect:
             path = redirect.path
-        return models.Subreddit(self, path.split('/')[2])
+        return models.Subreddit(self, path.split("/")[2])
 
     def redditor(self, name):
         """Return a lazy instance of :class:`~.Redditor` for ``name``.
@@ -468,15 +577,18 @@ class Reddit(object):
             (default: None).
 
         """
-        return self._core.request(method, path, data=data, files=files,
-                                  params=params)
+        return self._core.request(
+            method, path, data=data, files=files, params=params
+        )
 
     def submission(  # pylint: disable=invalid-name,redefined-builtin
-            self, id=None, url=None):
+        self, id=None, url=None
+    ):
         """Return a lazy instance of :class:`~.Submission`.
 
         :param id: A reddit base36 submission ID, e.g., ``2gmzqe``.
-        :param url: A URL supported by :meth:`.id_from_url`.
+        :param url: A URL supported by
+            :meth:`~praw.models.Submission.id_from_url`.`.
 
         Either ``id`` or ``url`` can be provided, but not both.
 
